#!/usr/local/bin/wish

############################################################
# Backhoe simulator
# $Id: backhoe.tcl 188 2006-12-13 01:56:34Z tbourke $
set version [string map {"Revision:" "" "\$" "" " " ""} {$Revision: 188 $}]
#
# 20051019 T.Bourke (UNSW CSE/NICTA)
#    Created original version for COMP3241/9245
#    The code for connecting to Xes comes from the INRIA/CMA
#    Esterel v5_92 distribution.
#
# References:
#   * `How Caterpillar Backhoe Loaders Work', Marshall Brain, Tom Harris
#     http://www.howstuffworks.com/backhoe-loader.htm
#   * John Deere 110TLB and 310G Backhoe loaders
#

############################################################
# Initialize globals

set vecscale 0.1
set xoffset -600
set yoffset -500

set butcolour lightgray
set butactive lightgray

############################################################
# Functions for working in 2D

proc lcoords {fn args ol} {
	set nl [list]
	for {set i 0} {$i < [llength $ol]} {incr i 2} {
		set xy [eval [list $fn $args [lrange $ol $i [expr $i + 1]]]]
		set nl [concat $nl $xy]
	}
	return $nl
}

# arg = 1: translate from coords to screen
# arg = 0: translate from screen to coords (inverse)
proc fixcoords {arg xy} {
	global vecscale xoffset yoffset
	set x [lindex $xy 0]
	set y [lindex $xy 1]
	if {$arg == 1} {
		return [list	[expr int(($x + $xoffset) * $vecscale)] \
						[expr int(($y + $yoffset) * $vecscale)]]
	} else {
		return [list	[expr int(($x / $vecscale) - $xoffset)] \
						[expr int(($y / $vecscale) - $yoffset)]]
	}
}

proc translate {args xy} {
	set x [lindex $xy 0]
	set y [lindex $xy 1]
	set offx [lindex $args 0]
	set offy [lindex $args 1]
	return [list [expr $x + $offx] [expr $y + $offy]]
}

proc rotate {args xy} {
	set x [lindex $xy 0]
	set y [lindex $xy 1]
	set theta [lindex $args 0]
	set offx [lindex $args 1]
	set offy [lindex $args 2]
	set xr [expr (($x - $offx) * cos($theta) - ($y - $offy) * sin($theta)) + $offx]
	set yr [expr (($x - $offx) * sin($theta) + ($y - $offy) * cos($theta)) + $offy]
	return [list $xr $yr]
}

proc pointdiff {fst snd} {
	set x [expr [lindex $fst 0] - [lindex $snd 0]]
	set y [expr [lindex $fst 1] - [lindex $snd 1]]
	return [list $x $y]
}

proc processcoords {fn arg} {
	global TractorCoords
	foreach n [array names TractorCoords] {
		set TractorCoords($n) [lcoords $fn $arg $TractorCoords($n)]
	}
}

proc rotatepoint {point pivotpoint angle} {
	return [rotate [concat [list $angle] $pivotpoint] $point]
}

proc rotatecoords {name pivotpoint angle} {
	global TractorCoords
	return [lcoords rotate [concat [list $angle] $pivotpoint] \
			$TractorCoords($name)]
}

############################################################
# Setup Backhoe data structures

set components [list loaderbucket underbelly frontshell loaderarm backshell \
				cabinstruts cabinroof cabininterior cabinsteer1 cabinsteer2 \
				cabinseat bucket stick boom mount legs]
set indicators [list Alarm Done Cancelled]

set Status(Paused) 0
set Status(Connected) 0

set PendingSignals [list]

proc initstatus {} {
	global Status ArmPos ArmMin ArmMax

	set ArmPos(boom) $ArmMin(boom)
	set ArmPos(stick) $ArmMax(stick)
	set ArmPos(bucket) $ArmMax(bucket)
	set ArmPos(legs) $ArmMin(legs)

	set Status(Alarm) 0
	set Status(Done) 0
	set Status(Cancelled) 0
	set Status(boomdir) 1
	set Status(boominc) 0
	set Status(stickdir) 1
	set Status(stickinc) 0
	set Status(bucketdir) 1
	set Status(bucketinc) 0
	set Status(legsdir) 1
	set Status(legsinc) 0

	set Status(React) 0
	set Status(StepCount) 0
}

set Option(PauseAfterReact) 0
set Option(SimStepDelay) 100
set Option(SimStepPerSECOND) 10
set Option(SendSecondInputs) 1
set Option(EspPath) {}

set Option(boomspeed) 0.4
set Option(stickspeed) 0.5
set Option(bucketspeed) 0.35
set Option(legsspeed) 0.4

set TractorCoords(loaderbucket) [list 1228 4204 992 3826 755 4204]
set TractorCoords(loaderarm) [list 1228 4204 1748 3685 1937 3685 2031 3543 \
								2362 3401 2362 3354 1700 3543 1181 4110]
set TractorCoords(frontshell) [list 2456 3307 1653 3543 1653 3732 1889 3742 \
								1889 3826 2362 3779 2362 3921 2456 3921 \
								2456 3826 2740 3826 2692 3779 2456 3779 \
								2456 3732 2409 3685 2409 3590 2456 3307]
set TractorCoords(backshell) [list 2740 3826 2787 3685 2881 3543 2976 3496 \
								3448 3543 3448 3496 3354 3401 2929 3401 \
								2787 3543 2692 3779 2740 3826]
set TractorCoords(underbelly) [list 1653 3732 1653 3921 2362 3921 2409 3968 \
									2503 3968 2503 4015 2692 4015 2929 3685 \
									3448 3685 3448 3496 3354 3401 2929 3401 \
									2787 3543 2692 3779 2645 3779]
set TractorCoords(cabinstruts) [list 2409 3307 2598 2692 3401 2692 3401 3501 \
									3307 3401 3307 2740 2645 2740 2456 3307]
set TractorCoords(cabinroof) [list 2503 2740 2503 2692 2598 2645 2929 2598 \
									3637 2645 3637 2740 2503 2740]
set TractorCoords(cabininterior) [list 2456 3307 2551 3259 2692 3401 2692 3496 \
									2598 3590 2645 3779 2456 3779 2456 3732 \
									2409 3685 2409 3590 2456 3307]
set TractorCoords(cabinsteer1) [list 2692 3259 2717 3240 2807 3334 2787 3354 \
									2692 3259]
set TractorCoords(cabinsteer2) [list 2730 3317 2717 3299 2658 3345 2680 3360]
set TractorCoords(cabinseat) [list 2834 3496 2881 3307 3212 3307 3259 3401]
set TractorCoords(bucketfix) [list 4900 3300]	;#pivot point
set TractorCoords(bucket) [list 4771 3259 4818 3118 4960 3259 4866 3401 \
								4582 3496 4440 3448 4393 3307 4299 2976 \
								4629 3259 4771 3259]
set TractorCoords(stickfix) [list 4270 2314]	;#pivot point
set TractorCoords(stick) [list 4157 2314 4724 3118 4913 3354 5007 3212 \
								4913 3070 4818 3070 4535 2598 4582 2456 \
								4299 2125 4110 2220 4157 2314]
set TractorCoords(boomfix) [list 4015 3821]		;#pivot point
set TractorCoords(boom) [list 4015 3921 4110 3921 4204 3732 4157 2881 \
								4393 2173 4251 2173 4157 2409 3874 2787 \
								3732 2787 3685 2881 3732 3023 3826 3118 \
								4015 3637 4015 3921]
set TractorCoords(mount) [list 3543 4015 3685 4062 4062 3968 4062 3732 \
								3874 3637 3637 3354 3448 3543 3448 3968 \
								3543 4015]
set TractorCoords(stickpiston) [list 4015 2503 4204 2220]
set TractorCoords(bucketpiston) [list 4535 2503 4866 3070]
set TractorCoords(boomrod) [list 4015 3637 3874 3543 3732 3023 3826 3118 \
							4015 3637]
set TractorCoords(stickrod) [list 3732 2787 3968 2503 4062 2503 3874 2787 \
							3732 2787]
set TractorCoords(legs) [list 3637 4062 3637 3543 3779 3590 3779 4062 \
						3637 4062 ]
set TractorCoords(Alarm) [lcoords fixcoords 0 [list 40 120]]
set TractorCoords(Done) [lcoords fixcoords 0 [list 40 170]]
set TractorCoords(Cancelled) [lcoords fixcoords 0 [list 40 220]]

set TractorColours(loaderbucket) black
set TractorColours(loaderarm) orange
set TractorColours(frontshell) yellow
set TractorColours(backshell) yellow
set TractorColours(underbelly) dimgray
set TractorColours(cabinstruts) black
set TractorColours(cabinroof) yellow
set TractorColours(cabininterior) black
set TractorColours(cabinsteer1) black
set TractorColours(cabinsteer2) black
set TractorColours(cabinseat) black
set TractorColours(bucket) black
set TractorColours(stick) orange
set TractorColours(boom) yellow
set TractorColours(mount) dimgray
set TractorColours(stickpiston) black
set TractorColours(bucketpiston) black
set TractorColours(boomrod) orange
set TractorColours(stickrod) orange
set TractorColours(legs) red
set TractorColours(Alarm) red
set TractorColours(Done) green
set TractorColours(Cancelled) green

set ArmMin(boom) 0
set ArmMax(boom) 25
set ArmMinSig(boom) IN
set ArmMaxSig(boom) OUT
set ArmMin(stick) -20
set ArmMax(stick) 0
set ArmMinSig(stick) OUT
set ArmMaxSig(stick) IN
set ArmMin(bucket) -20
set ArmMax(bucket) 0
set ArmMinSig(bucket) OUT
set ArmMaxSig(bucket) IN
set ArmMin(legs) 0
set ArmMax(legs) 25
set ArmMinSig(legs) IN
set ArmMaxSig(legs) OUT

############################################################
# Xes Connection Code (from the INRIA/CMA Esterel v5_92
# installation, and the accompanying wristwatch example)
#

namespace eval XesClient {
	variable client_socket
	set client_socket -1
}

proc XesClient::InitConnection  { {serverInfoFile xesport.esp} } {
	variable client_socket
	if {[file extension $serverInfoFile] != ".esp"} {
		append serverInfoFile ".esp"
	}
	if { [catch {open $serverInfoFile r} fd] } {
		# ERROR: cannot open file
		return -code error
	}
	while {[gets $fd line]} {
		if {[regexp {(\S+)\s+(\d+)} $line dummy host port]} {
			if {[catch {socket $host $port} client_socket]} {
				# ERROR: cannot open socket
				return -code error
			} else {
				return 
			}
		}
	}
	# ERROR: bad xesport.esp file
	return -code error
}

proc XesClient::SendCsimulEvent { { string {} } } {
	variable client_socket
	if {[catch {puts $client_socket $string; flush $client_socket} err]} {
		toggleconnect
		setmessage {Disconnected!}
	}
}

proc XesClient::CloseConnection {} {
	variable client_socket
	catch {close $client_socket}
}

proc XesConnect {askforfile {file ""}} {
	global Status canvas Option
	if {[string length $file] == 0} {
		set file $Option(EspPath)
	}

	if { $askforfile } {
		set	Status(Connected) 0
		XesClient::CloseConnection

		set file [tk_getOpenFile -defaultextension ".esp" \
			-filetypes [list {{Server Port} .esp} {{All files} *}] \
			-title "Server Port File" \
			-initialfile $file]

		if { $file == "" } {
			return
		}
	}

	# waiting for file existence
	# (if launched from command line option)
	setmessage "Attempting connection via $file..."
	set max_try 5
	for {set try 0} {$try < $max_try} {incr try} {
		if {[file exists $file]} {
			after 500
			break
		}
		after 1000
	}
	setmessage {}

	if {($try == $max_try) || [catch {XesClient::InitConnection $file}]} {
		if {[tk_dialog .err "Error" "Cannot connect to \"$file\"" \
			error 0 Continue Exit] } {
			exit 0
		}
		set Status(Connected) 0
	} else {
		set Status(Connected) 1
	}

	if {$Status(Connected)} {
		setbutton $canvas.app.connect 1 disconnect
	} else {
		setbutton $canvas.app.connect 0 connect
	}
}

############################################################
# About dialog box

proc circle {canvas cx cy r linewid linecol fillcol} {
    $canvas create oval [expr $cx - $r] [expr $cy - $r] \
		[expr $cx + $r] [expr $cy + $r] \
	    -outline $linecol -width $linewid -fill $fillcol
}

proc nictatext {canvas cx cy weight size text} {
    set outergreen "#004f7d"
    $canvas create text $cx $cy -fill $outergreen -justify center -text $text \
	-font [font create -family Helvetica -size $size -weight $weight]
}

proc textwide {canvas cx cy colour incr text font} {
    set totallen [font measure $font -displayof $canvas $text]
    set x [expr $cx - ($totallen * $incr / 2)]

    foreach char [split $text ""] {
	set charsize [font measure $font -displayof $canvas $char]
	$canvas create text [expr $x + (($charsize * $incr) / 2)] $cy \
	    -fill $colour -justify center -text $char -font $font
	set x [expr $x + ($charsize * $incr)]
    }
}

proc unswtext {canvas cx cy weight size text} {
    $canvas create text $cx $cy -justify center -text $text \
        -font [font create -family Helvetica -size $size -weight $weight]
}
	
proc aboutapp {} {
    global version

    if {[winfo exists .aboutdlg] == 0} {
	toplevel .aboutdlg -background white

	frame .aboutdlg.msg -width 600 -height 200 -background white
	frame .aboutdlg.msg.logos -width 300 -height 200 -background white
	pack .aboutdlg.msg.logos -side left

	# NICTA Logo
	canvas .aboutdlg.msg.logos.nicta -width 270 -height 80 \
	    -background white -highlightthickness 0
	set nlogo .aboutdlg.msg.logos.nicta

	set outergreen "#004f7d"
	set lightgreen "#dbf4e3"
	set nictagreen "#009100"

	circle $nlogo 42 40 37 2 $outergreen $lightgreen
	$nlogo create line 42 40 252 40 -width 2 -fill $outergreen
	circle $nlogo 42 40 12 2 $outergreen $nictagreen
	circle $nlogo 79 40 12 2 $outergreen $nictagreen
	circle $nlogo 252 40 12 2 $outergreen white

	textwide $nlogo 166 27 $outergreen 1.25 "NATIONAL" \
	    [font create -family Helvetica -size 22 -weight "bold"]
	nictatext $nlogo 166 53 "bold" 18 "ICT AUSTRALIA"
	nictatext $nlogo 215 67 "bold" 10 "LIMITED"

	# UNSW Logo
	canvas .aboutdlg.msg.logos.unsw -width 270 -height 80 \
	    -background white -highlightthickness 0
	set ulogo .aboutdlg.msg.logos.unsw

	unswtext $ulogo 135 19 "bold" 22 "THE UNIVERSITY OF"
	unswtext $ulogo 135 39 "bold" 22 "NEW SOUTH WALES"
	textwide $ulogo 69 60 black 1.2 "SYDNEY" \
	    [font create -family Helvetica -size 18 -weight "bold"]
	textwide $ulogo 185 60 black 1.2 "AUSTRALIA" \
	    [font create -family Helvetica -size 18 -weight "bold"]
	$ulogo create oval 116 58 120 62 -fill black

	# Text
	canvas .aboutdlg.msg.text -width 280 -height 70 \
	    -background white -highlightthickness 0
	set msgtext .aboutdlg.msg.text
	$msgtext create text 135 10 -justify center \
	    -text "Backhoe Simulator r$version" \
	    -fill blue -font [font create -family Times -size 26 -weight "bold"]
	$msgtext create text 135 32 -justify center \
	    -text "(c) 2005, 2006 T. Bourke and A. Sowmya" \
	    -fill blue -font [font create -family Times -size 12 -weight "bold"]
	$msgtext create text 135 47 -justify center \
	    -text "Uses Xes Connection routines" \
	    -fill blue -font [font create -family Times -size 12 -weight "bold"]
	$msgtext create text 135 60 -justify center \
	    -text "from INRIA/CMA Esterel v5_92" \
	    -fill blue -font [font create -family Times -size 12 -weight "bold"]

	# Put it together
	button .aboutdlg.ok -text Ok -default active -command {destroy .aboutdlg}
	pack $ulogo $nlogo -pady 5 -side top
	pack $msgtext -padx 10 -side right
	pack .aboutdlg.msg .aboutdlg.ok -side top

	wm withdraw .aboutdlg
	update
	wm geometry .aboutdlg
	wm transient .aboutdlg .
	wm title .aboutdlg {About Backhoe Simulator}
	wm deiconify .aboutdlg

    } else {
	tk_messageBox -type ok -title hello -message "Window already exists."
    }
}

############################################################
# Backhoe specific

proc drawtyres {} {
	global canvas
	set ro [list 2876 3585 3548 4257]
	set ri [list 3036 3745 3388 4087]	
	set fo [list 1748 3779 2220 4251]
	set fi [list 1873 3904 2095 4126]

	
	eval "$canvas create oval [lcoords fixcoords 1 $ro]\
		-fill black -outline black -tags rearouter"
	eval "$canvas create oval [join [lcoords fixcoords 1 $ro] " "]\
			-fill black -outline black -tags rearouter"
	eval "$canvas create oval [lcoords fixcoords 1 $ri]\
			-fill yellow -outline yellow -tags rearinner"
	eval "$canvas create oval [lcoords fixcoords 1 $fo]\
			-fill black -outline black -tags frontouter"
	eval "$canvas create oval [lcoords fixcoords 1 $fi]\
			-fill yellow -outline yellow -tags frontinner"
}

proc makeindicator {name} {
	global canvas TractorCoords

	set x [lindex $TractorCoords($name) 0]
	set y [lindex $TractorCoords($name) 1]
	
	set x1 [expr $x - 10]
	set y1 [expr $y - 10]
	set x2 [expr $x + 10]
	set y2 [expr $y + 10]
	set ty [expr $y + 20]

	$canvas create oval $x1 $y1 $x2 $y2 -fill dimgray -tags $name
	$canvas create text $x $ty -text $name
}

proc updatebackhoe {} {
	global canvas TractorCoords ArmPos indicators

	#Calculate the angles relative to the x axis.
	set boomangle [expr $ArmPos(boom)]
	set stickangle [expr ($ArmPos(boom) + $ArmPos(stick))]
	set bucketangle [expr ($stickangle + $ArmPos(bucket))]

	set boomangle [expr $boomangle / 10.000]
	set stickangle [expr $stickangle / 10.000]
	set bucketangle [expr $bucketangle / 10.000]

	#Rotate the stickfix point,
	#Calculate a translation between stickfix and stickfix2
	#Translate the bucketfix point
	set stickfix2 [rotatepoint $TractorCoords(stickfix) $TractorCoords(boomfix) \
								$boomangle]
	set sticktrans [pointdiff $stickfix2 $TractorCoords(stickfix)]
	set bucketfix2 [translate $TractorCoords(bucketfix) $sticktrans]

	#Rotate the bucketfix2 point about the new stickfix point
	#Calculate a translation between bucketfix and bucketfix3
	set bucketfix3 [rotatepoint $bucketfix2 $stickfix2 $stickangle]
	set buckettrans [pointdiff $bucketfix3 $TractorCoords(bucketfix)]

	#Rotate the boom
	eval "$canvas coords boom [rotatecoords boom $TractorCoords(boomfix) $boomangle]"
	#Rotate then translate the stick
	set newstickcoords [rotatecoords stick $TractorCoords(stickfix) $stickangle]
	eval "$canvas coords stick [lcoords translate $sticktrans $newstickcoords]"
	#Rotate then translate the bucket
	set newbucketcoords [rotatecoords bucket $TractorCoords(bucketfix) $bucketangle]
	eval "$canvas coords bucket [lcoords translate $buckettrans $newbucketcoords]"

	#Update the legs
	eval "$canvas coords legs [lcoords translate [list 0 $ArmPos(legs)] $TractorCoords(legs)]"

	#Update the indicators
	global canvas TractorColours Status
	foreach name $indicators {
		if {$Status($name) == 1} {
			set colour $TractorColours($name)
		} else {
			set colour dimgray
		}
		$canvas itemconfigure $name -fill $colour
	}

	update
}

proc setmessage {msg} {
	global canvas
	$canvas itemconfigure Message -text $msg
	update idletasks
}

proc movearms {} {
	global ArmPos ArmMin ArmMinSig ArmMax ArmMaxSig Status Option

	foreach arm [array name ArmPos] {
		set dir $Status([format "%s%s" $arm dir])
		set inc $Status([format "%s%s" $arm inc])
		set speed $Option([format "%s%s" $arm speed])
		set sig none

		if {$inc != 0} {
			set newpos [expr $ArmPos($arm) + ($dir * $inc * $speed)]
			if {$newpos > $ArmMax($arm)} {
				set newpos $ArmMax($arm)
				set sig $ArmMaxSig($arm)
			}
			if {$newpos < $ArmMin($arm)} {
				set newpos $ArmMin($arm)
				set sig $ArmMinSig($arm)
			}
			if {[string compare $sig none] != 0} {
				signal [format "%s_%s" [string toupper $arm] $sig]
			}
			set ArmPos($arm) $newpos
		}
	}
}

proc buttonpushed {which} {
	set sig [format "%s_BUTTON" [string toupper "$which"]]
	signal $sig
}

proc quitapp {} {
	exit
}

proc togglepause {} {
	global Status
	setpause [expr $Status(Paused) == 0?1:0]
}

proc setbutton {name onoff newtext} {
	global butcolour butactive
	if {$onoff} {
		$name configure -text $newtext -background red -activebackground tomato
	} else {
		$name configure -text $newtext -background $butcolour -activebackground $butactive
	}
}

proc setpause {value} {
	global canvas Status

	if {$value} {
		set Status(Paused) 1
		setbutton $canvas.app.pause 1 continue
		set butstate disabled
	} else {
		set Status(Paused) 0
		setbutton $canvas.app.pause 0 pause
		set butstate normal
	}

	#foreach c {stop extend retract} {
	#	$canvas.controls.$c configure -state $butstate
	#}
}

proc toggleconnect {} {
	global canvas Status
	
	if {$Status(Connected)} {
		XesClient::CloseConnection
		set Status(Connected) 0
		setbutton $canvas.app.connect 0 connect
	} else {
		XesConnect 1
	}
}

proc stepbutton {} {
	global canvas Status
	if {[$canvas.options.step cget -state]} {
		set Status(PauseAfterReact) 0
		$canvas.options.deselect
	} else {
		set Status(PauseAfterReact) 1
		$canvas.options.select
	}
}

proc reset {} {
	global Status PendingSignals
	initstatus
	updatebackhoe
	BACKHOE_reset
	if {$Status(Connected)} {
		XesClient::SendCsimulEvent "!reset;"
	}
	set PendingSignals [list]
}

############################################################
# Signals

proc signal {sig} {
	global Status PendingSignals
	$sig
	if {[lsearch $PendingSignals $sig] == -1} {
		lappend PendingSignals $sig
	}
	set Status(React) 1
}

proc reaction {} {
	global Status PendingSignals
	BACKHOE
	if {$Status(Connected)} {
		XesClient::SendCsimulEvent [format "%s;" [join $PendingSignals " "]]
	}
	set PendingSignals [list]
	set Status(React) 0
}

############################################################
# Setup display

initstatus

# window and contents
wm title . {Backhoe Simulator}
frame .main
pack .main -side top -fill x
canvas .main.canvas -width 800 -height 600 -background lightblue
pack .main.canvas -side right
set canvas .main.canvas

# translate from tractor points into display points
processcoords fixcoords 1

# draw the ground and a message box
$canvas create rect 0 375 800 600 -fill peru -outline peru
$canvas create text 400 10 -text "Backhoe Simulator v$version" -tags Message
$canvas create text 200 580 -text "" -tags Status
after 5000 {$canvas itemconfigure Message -text ""}

# draw the backhoe loader
foreach i $components {
	eval ".main.canvas create poly $TractorCoords($i) -fill $TractorColours($i) -tags $i"
}
drawtyres

# draw the indicators
foreach i $indicators {
	makeindicator $i
}

# add the controller buttons
set f [frame $canvas.controls -bd 0 -background lightblue]
$canvas create window 0 0 -anchor nw -window $f
button $canvas.controls.stop -width 10 -text {stop} -command \
	{buttonpushed stop}
button $canvas.controls.extend -width 10 -text extend \
			-command {buttonpushed extend}
button $canvas.controls.retract -width 10 -text retract \
			-command {buttonpushed retract}
pack	$canvas.controls.stop $canvas.controls.extend \
		$canvas.controls.retract -side top

# add other buttons
set f [frame $canvas.app -bd 0]
$canvas create window 800 600 -anchor se -window $f
label $canvas.app.label -width 18 -text {Simulation controls:}

frame $canvas.app.admin -width 14 -bd 0
button $canvas.app.admin.quit -text {quit} -width 5 -command {quitapp}
button $canvas.app.admin.about -text {about} -width 5 -command {aboutapp}
pack $canvas.app.admin.quit $canvas.app.admin.about -side right

button $canvas.app.reset -text {reset} -width 14 -command {reset}
button $canvas.app.connect -text {connect} -width 14 -command {toggleconnect}
button $canvas.app.pause -text {pause} -width 14 -command {togglepause}
set butcolour [$canvas.app.pause cget -background]
set butactive [$canvas.app.pause cget -activebackground]
pack	$canvas.app.admin $canvas.app.reset $canvas.app.connect \
	$canvas.app.pause $canvas.app.label  -side bottom 

set f [frame $canvas.options -bd 0]
$canvas create window 0 600 -anchor sw -window $f
label $canvas.options.label -width 20 -text {Simulation options:}
checkbutton $canvas.options.secs -text {send SECOND inputs}\
	-width 20 -variable Option(SendSecondInputs)
checkbutton $canvas.options.step -text {pause after reactions}\
	-width 20 -variable Option(PauseAfterReact)
pack $canvas.options.secs $canvas.options.step $canvas.options.label -side bottom 

tkwait visibility .main

# go

setpause 1

if {$argc > 0} {
	set arg0 [lindex $argv 0]
	if {[string equal -length 8 {-client:} $arg0]} {
		set Option(EspPath) [string range $arg0 8 end]
		if {[catch {XesConnect 0}]} {
			puts stderr "Connection using $Option(EspPath) failed."
			exit 1
		}
	}
}

while {1} {
	global Status Option

	# Update the arms and indicators
	movearms	
	updatebackhoe

	incr Status(StepCount) 1
	if {$Status(StepCount) == $Option(SimStepPerSECOND)} {
		if {$Option(SendSecondInputs)} {
			signal SECOND
		}
		set Status(StepCount) 0
	}

	if {$Status(Paused)} {
		vwait Status(Paused)
	}
	if {$Status(React)} {
		# The boom, stick and bucket drive signals must be sustained
		foreach inc {boominc stickinc bucketinc} { set Status($inc) 0 }

		reaction
		if {$Option(PauseAfterReact)} {
			setpause 1
		}
	}

	after $Option(SimStepDelay)
	update
}

